<?php 
/*
Plugin Name: WooCommerce Serial Key
Plugin URI: http://www.storeapps.org/product/woocommerce-serial-key/
Description: Automatically create, issue & manage serial keys with product purchases. Easy for both, you & customer.
Version: 1.6
Author: Store Apps
Author URI: http://www.storeapps.org/
Copyright (c) 2013 Store Apps All rights reserved.
*/

if ( !defined( 'ABSPATH' ) ) exit;

$active_plugins = (array) get_option( 'active_plugins', array() );

if ( is_multisite() ) {
    $active_plugins = array_merge( $active_plugins, get_site_option( 'active_sitewide_plugins', array() ) );
}

if ( ! ( in_array( 'woocommerce/woocommerce.php', $active_plugins ) || array_key_exists( 'woocommerce/woocommerce.php', $active_plugins ) ) ) return;

register_activation_hook ( __FILE__, 'woocommerce_serial_key_activate' );

function woocommerce_serial_key_activate() {
	global $wpdb;

	$collate = '';

	if ( $wpdb->has_cap( 'collation' ) ) {
		if ( ! empty($wpdb->charset ) ) {
			$collate .= "DEFAULT CHARACTER SET $wpdb->charset";
		}
		if ( ! empty($wpdb->collate ) ) {
			$collate .= " COLLATE $wpdb->collate";
		}
	}

	require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );

	$create_table_query = "
				CREATE TABLE IF NOT EXISTS `{$wpdb->prefix}woocommerce_serial_key` (
				  `order_id` bigint(20) unsigned NOT NULL default '0',
				  `product_id` bigint(20) unsigned NOT NULL default '0',
				  `serial_key` varchar(50),
				  `valid_till` datetime,
				  KEY `product_id` (`product_id`),
				  KEY `order_id` (`order_id`)
				  ) $collate";
	
	dbDelta( $create_table_query );

	if ( get_option( 'sa_serial_key_db_version' ) === false ) {

    	sa_database_update_for_serial_key_1_3();		
	}

}

function sa_database_update_for_serial_key_1_3() {
	global $wpdb;

	if ( $wpdb->get_var( "SHOW TABLES LIKE '{$wpdb->prefix}woocommerce_serial_key';" ) ) {
		if ( ! $wpdb->get_var( "SHOW COLUMNS FROM `{$wpdb->prefix}woocommerce_serial_key` LIKE 'limit';" ) ) {
			$wpdb->query( "ALTER TABLE {$wpdb->prefix}woocommerce_serial_key ADD `limit` int(11) UNSIGNED NOT NULL DEFAULT 0;" );
		}
		if ( ! $wpdb->get_var( "SHOW COLUMNS FROM `{$wpdb->prefix}woocommerce_serial_key` LIKE 'uuid';" ) ) {
			$wpdb->query( "ALTER TABLE {$wpdb->prefix}woocommerce_serial_key ADD `uuid` longtext NOT NULL DEFAULT '';" );
		}
	}

	update_option( 'sa_serial_key_db_version', '1.3' );

}

function initialize_woocommerce_serial_key() {

	require_once 'classes/class-wc-compatibility.php';
	require_once 'classes/class-wc-compatibility-2-2.php';
	require_once 'classes/class-wc-compatibility-2-3.php';

	if ( !class_exists( 'SA_Serial_Key' ) ) {

		require_once 'classes/class-sa-serial-key.php';

	}

	$GLOBALS['sa_serial_key'] = new SA_Serial_Key( __FILE__ );

	if ( !class_exists( 'Validate_Serial_Key' ) ) {

		require_once 'classes/class-validate-serial-key.php';

	}

	if ( !class_exists( 'Store_Apps_Upgrade' ) ) {

		require_once 'sa-includes/class-storeapps-upgrade.php';

	}

	$sku = 'wcsk';
	$prefix = 'sa_serial_key';
	$plugin_name = 'WooCommerce Serial Key';
	$text_domain = 'localize_woocommerce_serial_key';
	$documentation_link = 'http://www.storeapps.org/support/documentation/woocommerce-serial-key/';
	$GLOBALS['serial_key_upgrade'] = new Store_Apps_Upgrade( __FILE__, $sku, $prefix, $plugin_name, $text_domain, $documentation_link );

}
add_action( 'plugins_loaded', 'initialize_woocommerce_serial_key' );